package live_logic

import (
	pbCommon "dcg/game/pb/common"
	pbMq "dcg/game/pb/mq"
	"dcg/pkg/cmd"
)

type (
	LiveLogic interface {
		SetGameType(gameType string)
		GameType() string

		SetCmdParser(parser *cmd.Parser)
		SetPreDanmakuHandler(h PreDanmakuHandlerFunc)
		RegisterCMDHandler(h CMDHandlerFunc, cmd string, alias ...string)
		RegisterGiftHandler(h GiftHandler)
		RegisterNobilityHandler(h NobilityHandlerFunc)

		HandleDanmaku(liveRoom *LiveRoom, user *pbCommon.PbUser, dm *pbMq.MqDanmaku)
		HandleGift(liveRoom *LiveRoom, user *pbCommon.PbUser, gift *pbMq.MqGift)
		HandleNobility(liveRoom *LiveRoom, user *pbCommon.PbUser, msg *pbMq.MqNobilityBuy)
	}
	// LiveRoom 直播间定位
	LiveRoom struct {
		RoomId   int64
		Platform string
	}

	PreDanmakuHandlerResp struct {
		PushDanmaku bool // 之后的处理逻辑是否继续push普通danmaku
		Parsing     bool // 之后是否继续处理命令解析
	}

	PreDanmakuHandlerFunc func(liveRoom *LiveRoom, user *pbCommon.PbUser, dm *pbMq.MqDanmaku) PreDanmakuHandlerResp
	CMDHandlerFunc        func(liveRoom *LiveRoom, prefix string, content []rune, user *pbCommon.PbUser)
	GiftHandlerFunc       func(liveRoom *LiveRoom, user *pbCommon.PbUser, gift *pbMq.MqGift)
	GiftHandler           func(next GiftHandlerFunc) GiftHandlerFunc

	NobilityHandlerFunc func(liveRoom *LiveRoom, user *pbCommon.PbUser, nobility *pbMq.MqNobilityBuy)
	// GiftLogic LiveLogic内部礼物逻辑处理
	GiftLogic struct {
		GiftIds []int64
		Fn      GiftLogicFunc
	}
	GiftLogicFunc func(liveRoom *LiveRoom, user *pbCommon.PbUser, gift *pbMq.MqGift)
)

var EmptyGiftHandlerFunc GiftHandlerFunc = func(liveRoom *LiveRoom, user *pbCommon.PbUser, gift *pbMq.MqGift) {}
var DefaultPreDanmakuHandlerFunc PreDanmakuHandlerFunc = func(liveRoom *LiveRoom, user *pbCommon.PbUser, dm *pbMq.MqDanmaku) PreDanmakuHandlerResp {
	return PreDanmakuHandlerResp{
		PushDanmaku: true,
		Parsing:     true,
	}
}
