package game

import (
	"dcg/game/manager"
	pbCommon "dcg/game/pb/common"
	pbGameZhgWW2 "dcg/game/pb/game/zhgww2"
	"dcg/game/svc"
	"git.noahlan.cn/northlan/ngs/component"
	"git.noahlan.cn/northlan/ngs/session"
	"git.noahlan.cn/northlan/ntools-go/logger"
)

type GameLogic struct {
	component.Base
	svcCtx *svc.ServiceContext
}

func NewGameLogic(svcCtx *svc.ServiceContext) *GameLogic {
	return &GameLogic{
		svcCtx: svcCtx,
	}
}

func (p *GameLogic) Init() {
}

func (p *GameLogic) Shutdown() {
}

func (p *GameLogic) CMD() string {
	return "game"
}

func (p *GameLogic) Prefix() string {
	return ""
}

// Status 游戏房间状态变更
func (p *GameLogic) Status(s *session.Session, msg *pbCommon.GameStatusReq) error {
	room, err := manager.GameManager.RoomBySession(s)
	if err != nil {
		return s.Response(&pbCommon.GameStatusResp{
			Success: false,
		})
	}
	room.SetStatus(manager.GameStatus(msg.Status))

	return s.Response(&pbCommon.GameStatusResp{
		Success:   true,
		BattleId:  room.BattleId,
		Timestamp: msg.Timestamp,
	})
}

// Join 玩家加入游戏
func (p *GameLogic) Join(s *session.Session, msg *pbGameZhgWW2.JoinGameReq) error {
	logger.SLog.Infof("玩家 [%s] 加入游戏-客户端请求", msg.User.Username)

	room, err := manager.GameManager.RoomBySession(s)
	if err != nil {
		return s.Response(&pbGameZhgWW2.JoinGameResp{
			User:     msg.User,
			BattleId: msg.BattleId,
			Team:     msg.Team,
			Success:  false,
			Message:  "未找到游戏房间",
		})
	}
	room.UserJoin(manager.User{
		ID:       msg.User.UserId,
		Username: msg.User.Username,
		Platform: msg.User.Platform,
		Avatar:   msg.User.Avatar,
		Team:     msg.Team,
	})

	return s.Response(&pbGameZhgWW2.JoinGameResp{
		User:     msg.User,
		BattleId: msg.BattleId,
		Team:     msg.Team,
	})
}
